@@ -1,12 +1,10 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-""" erppeek.py -- OpenERP command line tool
+""" erppeek.py -- Odoo / OpenERP client library and command line tool
 
 Author: Florent Xicluna
 (derived from a script by Alan Bell)
 """
-from __future__ import with_statement
-
 import _ast
 import atexit
 import collections
@@ -16,10 +14,10 @@ import optparse
 import os
 from pprint import pprint
 import re
+import shlex
 import sys
 import time
 import traceback
-import warnings
 try:                    # Python 3
     import configparser
     from threading import current_thread
@@ -47,9 +45,9 @@ except ImportError:     # Python 2
                     for cell in rowlst]
 
 
-__version__ = '1.5.3'
+__version__ = '1.6'
 __all__ = ['Client', 'Model', 'Record', 'RecordList', 'Service',
-           'format_exception', 'read_config', 'start_openerp_services']
+           'format_exception', 'read_config', 'start_odoo_services']
 
 CONF_FILE = 'erppeek.ini'
 HIST_FILE = os.path.expanduser('~/.erppeek_history')
@@ -83,8 +81,8 @@ Usage (some commands):
 STABLE_STATES = ('uninstallable', 'uninstalled', 'installed')
 DOMAIN_OPERATORS = frozenset('!|&')
 # Supported operators are:
-#   =, !=, >, >=, <, <=, like, ilike, in, not like, not ilike, not in, child_of
-#   =like, =ilike (6.0), =? (6.0)
+#   =, !=, >, >=, <, <=, like, ilike, in, not like, not ilike, not in,
+#   child_of, =like, =ilike, =?
 _term_re = re.compile(
     '([\w._]+)\s*'  '(=(?:like|ilike|\?)|[<>]=?|!?=(?!=)'
     '|(?<= )(?:like|ilike|in|not like|not ilike|not in|child_of))'  '\s*(.*)')
@@ -92,24 +90,27 @@ _fields_re = re.compile(r'(?:[^%]|^)%\(([^)]+)\)')
 
 # Published object methods
 _methods = {
-    'db': ['create', 'drop', 'dump', 'restore', 'rename',
-           'get_progress', 'list', 'list_lang',
+    'db': ['create_database', 'duplicate_database', 'db_exist',
+           'drop', 'dump', 'restore', 'rename', 'list', 'list_lang',
            'change_admin_password', 'server_version', 'migrate_databases'],
-    'common': ['about', 'login', 'timezone_get', 'get_server_environment',
-               'login_message', 'check_connectivity'],
-    'object': ['execute', 'exec_workflow'],
-    'report': ['report', 'report_get'],
-    'wizard': ['execute', 'create'],
+    'common': ['about', 'login', 'timezone_get',
+               'authenticate', 'version', 'set_loglevel'],
+    'object': ['execute', 'execute_kw', 'exec_workflow'],
+    'report': ['render_report', 'report', 'report_get'],
 }
-_methods_6_1 = {
-    'db': ['create_database', 'db_exist'],
-    'common': ['get_stats', 'list_http_services', 'version',
-               'authenticate', 'get_os_time', 'get_sqlcount'],
-    'object': ['execute_kw'],
-    'report': ['render_report'],
+# New 6.1: (db) create_database db_exist,
+#          (common) authenticate version set_loglevel
+#          (object) execute_kw,  (report) render_report
+# New 7.0: (db) duplicate_database
+
+_obsolete_methods = {
+    'db': ['create', 'get_progress'],                   # < 8.0
+    'common': ['check_connectivity', 'get_available_updates', 'get_os_time',
+               'get_migration_scripts', 'get_server_environment',
+               'get_sqlcount', 'get_stats',
+               'list_http_services', 'login_message'],  # < 8.0
+    'wizard': ['execute', 'create'],                    # < 7.0
 }
-# Hidden methods:
-#  - common: get_available_updates, get_migration_scripts, set_loglevel
 _cause_message = ("\nThe above exception was the direct cause "
                   "of the following exception:\n\n")
 
@@ -183,8 +184,10 @@ def format_exception(exc_type, exc, tb, limit=None, chain=True,
     If `chain` is True, then the original exception is printed too.
     """
     values = _format_exception(exc_type, exc, tb, limit=limit)
-    if ((issubclass(exc_type, Fault) and
-         isinstance(exc.faultCode, basestring))):
+    if issubclass(exc_type, Error):
+        values = [str(exc) + '\n']
+    elif ((issubclass(exc_type, Fault) and
+           isinstance(exc.faultCode, basestring))):
         # Format readable 'Fault' errors
         (etype, __, msg) = exc.faultCode.partition('--')
         server_tb = None
@@ -220,40 +223,46 @@ def read_config(section=None):
     env = dict(p.items(section))
     scheme = env.get('scheme', 'http')
     if scheme == 'local':
-        server = (scheme, env.get('options', ''))
+        server = shlex.split(env.get('options', ''))
     else:
         server = '%s://%s:%s' % (scheme, env['host'], env['port'])
     return (server, env['database'], env['username'], env.get('password'))
 
 
-def start_openerp_services(options=None, appname=None):
-    """Initialize the OpenERP services.
+def start_odoo_services(options=None, appname=None):
+    """Initialize the Odoo services.
 
-    Import the ``openerp`` package and load the OpenERP services.
-    The argument `options` receives the command line arguments for ``openerp``.
-    Example: ``"-c /path/to/openerp-server.conf --without-demo all"``.
-    Return the openerp module.
+    Import the ``openerp`` package and load the Odoo services.
+    The argument `options` receives the command line arguments
+    for ``openerp``.  Example:
+      ``['-c', '/path/to/openerp-server.conf', '--without-demo', 'all']``.
+    Return the ``openerp`` package.
     """
-    import openerp
-    if not openerp.osv.osv.service:
+    import openerp as odoo
+    odoo._api_v7 = odoo.release.version_info < (8,)
+    if not (odoo._api_v7 and odoo.osv.osv.service):
         os.environ['TZ'] = 'UTC'
         if appname is not None:
             os.environ['PGAPPNAME'] = appname
-        options = options.split() if options else []
-        openerp.tools.config.parse_config(options)
-        if openerp.release.version_info < (7,):
-            openerp.netsvc.init_logger()
-            openerp.osv.osv.start_object_proxy()
-            openerp.service.web_services.start_web_services()
-        else:
-            openerp.service.start_internal()
+        odoo.tools.config.parse_config(options or [])
+        if odoo.release.version_info < (7,):
+            odoo.netsvc.init_logger()
+            odoo.osv.osv.start_object_proxy()
+            odoo.service.web_services.start_web_services()
+        elif odoo._api_v7:
+            odoo.service.start_internal()
+        else:   # Odoo v8
+            try:
+                odoo.api.Environment._local.environments = odoo.api.Environments()
+            except AttributeError:
+                pass
 
         def close_all():
-            for db in openerp.modules.registry.RegistryManager.registries:
-                openerp.sql_db.close_db(db)
+            for db in odoo.modules.registry.RegistryManager.registries:
+                odoo.sql_db.close_db(db)
         atexit.register(close_all)
 
-    return openerp
+    return odoo
 
 
 def issearchdomain(arg):
@@ -264,10 +273,7 @@ def issearchdomain(arg):
       - ``['name = mushroom', 'state != draft']``
       - ``[]``
     """
-    # These ones are supported but discouraged:
-    # - 'state != draft'
-    # - ('state', '!=', 'draft')
-    return isinstance(arg, (list, tuple, basestring)) and not (arg and (
+    return isinstance(arg, list) and not (arg and (
         # Not a list of ids: [1, 2, 3]
         isinstance(arg[0], int_types) or
         # Not a list of ids as str: ['1', '2', '3']
@@ -279,10 +285,7 @@ def searchargs(params, kwargs=None, context=None):
     if not params:
         return ([],)
     domain = params[0]
-    if isinstance(domain, (basestring, tuple)):
-        domain = [domain]
-        warnings.warn('Domain should be a list: %s' % domain)
-    elif not isinstance(domain, list):
+    if not isinstance(domain, list):
         return params
     for (idx, term) in enumerate(domain):
         if isinstance(term, basestring) and term not in DOMAIN_OPERATORS:
@@ -307,6 +310,10 @@ def searchargs(params, kwargs=None, context=None):
     return params
 
 
+class Error(Exception):
+    """An ERPpeek error."""
+
+
 class Service(object):
     """A wrapper around XML-RPC endpoints.
 
@@ -318,6 +325,8 @@ class Service(object):
     which should be exposed on this endpoint.  Use ``dir(...)`` on the
     instance to list them.
     """
+    _rpcpath = ''
+
     def __init__(self, server, endpoint, methods, verbose=False):
         if isinstance(server, basestring):
             self._rpcpath = rpcpath = server + '/xmlrpc/'
@@ -325,10 +334,12 @@ class Service(object):
             self._dispatch = proxy._ServerProxy__request
             if hasattr(proxy._ServerProxy__transport, 'close'):   # >= 2.7
                 self.close = proxy._ServerProxy__transport.close
-        else:
-            self._rpcpath = ''
+        elif server._api_v7:
             proxy = server.netsvc.ExportService.getService(endpoint)
             self._dispatch = proxy.dispatch
+        else:   # Odoo v8
+            self._dispatch = functools.partial(server.http.dispatch_rpc,
+                                               endpoint)
         self._endpoint = endpoint
         self._methods = methods
         self._verbose = verbose
@@ -376,7 +387,7 @@ class Service(object):
 
 
 class Client(object):
-    """Connection to an OpenERP instance.
+    """Connection to an Odoo instance.
 
     This is the top level object.
     The `server` is the URL of the instance, like ``http://localhost:8069``.
@@ -387,34 +398,37 @@ class Client(object):
     table ``res.users``.  If the `password` is not provided, it will be
     asked on login.
     """
-    _config_file = os.path.join(os.path.curdir, CONF_FILE)
+    _config_file = os.path.join(os.curdir, CONF_FILE)
 
     def __init__(self, server, db=None, user=None, password=None,
                  verbose=False):
         if isinstance(server, basestring) and server[-1:] == '/':
             server = server.rstrip('/')
+        elif isinstance(server, list):
+            appname = os.path.basename(__file__).rstrip('co')
+            server = start_odoo_services(server, appname=appname)
         self._server = server
-        major_version = None
+        float_version = 999
 
         def get_proxy(name):
-            if major_version in ('5.0', None) or name == 'wizard':
-                methods = _methods[name]
-            else:
-                # Only for OpenERP >= 6
-                methods = _methods[name] + _methods_6_1[name]
+            methods = list(_methods[name]) if (name in _methods) else []
+            if float_version < 8.0:
+                methods += _obsolete_methods.get(name) or ()
             return Service(server, name, methods, verbose=verbose)
         self.server_version = ver = get_proxy('db').server_version()
-        self.major_version = major_version = '.'.join(ver.split('.', 2)[:2])
+        self.major_version = re.match('\d+\.?\d*', ver).group()
+        float_version = float(self.major_version)
         # Create the XML-RPC proxies
         self.db = get_proxy('db')
         self.common = get_proxy('common')
         self._object = get_proxy('object')
         self._report = get_proxy('report')
-        self._wizard = get_proxy('wizard') if major_version < '7.0' else None
+        self._wizard = get_proxy('wizard') if float_version < 7.0 else None
         self.reset()
+        self.context = None
         if db:
             # Try to login
-            self._login(user, password=password, database=db)
+            self.login(user, password=password, database=db)
 
     @classmethod
     def from_config(cls, environment, verbose=False):
@@ -425,11 +439,9 @@ class Client(object):
         See :func:`read_config` for details of the configuration file format.
         """
         (server, db, user, password) = read_config(environment)
-        if server[0] == 'local':
-            appname = os.path.basename(__file__).rstrip('co')
-            server = start_openerp_services(server[1], appname=appname)
-        client = cls(server, db, user, password, verbose=verbose)
+        client = cls(server, verbose=verbose)
         client._environment = environment
+        client.login(user, password=password, database=db)
         return client
 
     def reset(self):
@@ -446,24 +458,26 @@ class Client(object):
         If the `password` is not available, it will be asked.
         """
         if database:
-            dbs = self.db.list()
-            if database not in dbs:
-                print("Error: Database '%s' does not exist: %s" %
-                      (database, dbs))
-                return
+            try:
+                dbs = self.db.list()
+            except Fault:
+                pass    # AccessDenied: simply ignore this check
+            else:
+                if database not in dbs:
+                    raise Error("Database '%s' does not exist: %s" %
+                                (database, dbs))
+            if not self._db:
+                self._db = database
+            # Used for logging, copied from openerp.sql_db.db_connect
+            current_thread().dbname = database
         elif self._db:
             database = self._db
         else:
-            print('Error: Not connected')
-            return
-        # Used for logging, copied from openerp.sql_db.db_connect
-        current_thread().dbname = database
+            raise Error('Not connected')
         (uid, password) = self._auth(database, user, password)
         if not uid:
-            if not self._db:
-                self._db = database
-            print('Error: Invalid username or password')
-            return
+            current_thread().dbname = self._db
+            raise Error('Invalid username or password')
         if self._db != database:
             self.reset()
             self._db = database
@@ -477,7 +491,7 @@ class Client(object):
         self.report = authenticated(self._report.report)
         self.report_get = authenticated(self._report.report_get)
         if self.major_version != '5.0':
-            # Only for OpenERP >= 6
+            # Only for Odoo and OpenERP >= 6
             self.execute_kw = authenticated(self._object.execute_kw)
             self.render_report = authenticated(self._report.render_report)
         if self._wizard:
@@ -548,6 +562,7 @@ class Client(object):
             """Return a model registry.
 
             Use get_pool(db_name).db.cursor() to grab a cursor.
+            With Odoo v8, use get_pool(db_name).cursor() instead.
             """
             client = global_vars['client']
             registry = client._server.modules.registry
@@ -559,9 +574,13 @@ class Client(object):
                 # Safety measure: turn down the previous connection
                 global_vars['client'].reset()
                 client = self.from_config(env, verbose=self.db._verbose)
-            else:
-                client = self
-                env = self._environment or self._db
+                return
+            client = self
+            env = client._environment or client._db
+            try:  # copy the context to the new client
+                client.context = dict(global_vars['client'].context)
+            except (KeyError, TypeError):
+                pass  # client not yet in globals(), or context is None
             global_vars['client'] = client
             if hasattr(client._server, 'modules'):
                 global_vars['get_pool'] = get_pool
@@ -579,7 +598,11 @@ class Client(object):
 
         def login(self, user, password=None, database=None):
             """Switch `user` and (optionally) `database`."""
-            if self._login(user, password=password, database=database):
+            try:
+                self._login(user, password=password, database=database)
+            except Error as exc:
+                print('%s: %s' % (exc.__class__.__name__, exc))
+            else:
                 # Register the new globals()
                 self.connect()
 
@@ -597,18 +620,19 @@ class Client(object):
         By default, `demo` data are not loaded and `lang` is ``en_US``.
         Wait for the thread to finish and login if successful.
         """
-        thread_id = self.db.create(passwd, database, demo, lang, user_password)
-        progress = 0
-        try:
-            while progress < 1:
-                time.sleep(3)
-                progress, users = self.db.get_progress(passwd, thread_id)
-            # [1.0, [{'login': 'admin', 'password': 'admin',
-            #         'name': 'Administrator'}]]
-            self.login(users[0]['login'], users[0]['password'],
-                       database=database)
-        except KeyboardInterrupt:
-            print({'id': thread_id, 'progress': progress})
+        if self.major_version in ('5.0', '6.0'):
+            thread_id = self.db.create(passwd, database, demo, lang, user_password)
+            progress = 0
+            try:
+                while progress < 1:
+                    time.sleep(3)
+                    progress, users = self.db.get_progress(passwd, thread_id)
+            except KeyboardInterrupt:
+                return {'id': thread_id, 'progress': progress}
+        else:
+            self.db.create_database(passwd, database, demo, lang,
+                                    user_password)
+        return self.login('admin', user_password, database=database)
 
     def execute(self, obj, method, *params, **kwargs):
         """Wrapper around ``object.execute`` RPC method.
@@ -623,7 +647,7 @@ class Client(object):
         assert isinstance(method, basestring) and method != 'browse'
         context = kwargs.pop('context', None)
         ordered = single_id = False
-        if method in ('read', 'name_get'):
+        if method == 'read':
             assert params
             if issearchdomain(params[0]):
                 # Combine search+read
@@ -644,10 +668,8 @@ class Client(object):
                 return ids
             if len(params) > 1:
                 params = (ids,) + params[1:]
-            elif method == 'read':
-                params = (ids, kwargs.pop('fields', None))
             else:
-                params = (ids,)
+                params = (ids, kwargs.pop('fields', None))
         elif method == 'search':
             # Accept keyword arguments for the search method
             params = searchargs(params, kwargs, context)
@@ -704,6 +726,8 @@ class Client(object):
             if action == 'init' and name != wiz_id:
                 return wiz_id
             datas = {}
+        if context is None:
+            context = self.context
         return self._wizard_execute(wiz_id, datas, action, context)
 
     def _upgrade(self, modules, button):
@@ -724,9 +748,8 @@ class Client(object):
                 print('Already up-to-date: %s' %
                       self.modules([('id', 'in', ids)]))
             elif modules:
-                print('Module(s) not found: %s' % ', '.join(modules))
-            else:
-                print('%s module(s) updated' % updated)
+                raise Error('Module(s) not found: %s' % ', '.join(modules))
+            print('%s module(s) updated' % updated)
             return
         print('%s module(s) selected' % len(ids))
         print('%s module(s) to process:' % len(mods))
@@ -863,7 +886,7 @@ class Client(object):
             errmsg = 'Model not found.  These models exist:'
         else:
             errmsg = 'Model not found: %s' % (name,)
-        print('\n * '.join([errmsg] + [str(m) for m in models.values()]))
+        raise Error('\n * '.join([errmsg] + [str(m) for m in models.values()]))
 
     def modules(self, name='', installed=None):
         """Return a dictionary of modules.
@@ -935,7 +958,7 @@ class Client(object):
 
 
 class Model(object):
-    """The class for OpenERP models."""
+    """The class for Odoo models."""
 
     def __new__(cls, client, name):
         return client.model(name)
@@ -998,8 +1021,10 @@ class Model(object):
         or a search domain.
         If it is a single integer, the return value is a :class:`Record`.
         Otherwise, the return value is a :class:`RecordList`.
+        Be careful when passing a list of ids, because an empty list will be
+        considered an empty domain and will find all records in the database.
         """
-        context = kwargs.pop('context', None)
+        context = kwargs.pop('context', self.client.context)
         if isinstance(domain, int_types):
             assert not params and not kwargs
             return Record(self, domain, context=context)
@@ -1021,6 +1046,8 @@ class Model(object):
         :class:`Record` or None.  If multiple records are found,
         a ``ValueError`` is raised.
         """
+        if context is None:
+            context = self.client.context
         if isinstance(domain, int_types):   # a single id
             return Record(self, domain, context=context)
         if isinstance(domain, basestring):  # lookup the xml_id
@@ -1041,8 +1068,15 @@ class Model(object):
         """Create a :class:`Record`.
 
         The argument `values` is a dictionary of values which are used to
-        create the record.  The newly created :class:`Record` is returned.
+        create the record.  Relationship fields `one2many` and `many2many`
+        accept either a list of ids or a RecordList or the extended Odoo
+        syntax.  Relationship fields `many2one` and `reference` accept
+        either a Record or the Odoo syntax.
+
+        The newly created :class:`Record` is returned.
         """
+        if context is None:
+            context = self.client.context
         values = self._unbrowse_values(values)
         new_id = self._execute('create', values, context=context)
         return Record(self, new_id, context=context)
@@ -1070,7 +1104,7 @@ class Model(object):
             elif value and field_type == 'reference':
                 (res_model, res_id) = value.split(',')
                 rel_model = self.client.model(res_model, False)
-                values[key] = Record(rel_model, int(res_id))
+                values[key] = Record(rel_model, int(res_id), context=context)
         return values
 
     def _unbrowse_values(self, values):
@@ -1118,6 +1152,8 @@ class Model(object):
 
         def wrapper(self, *params, **kwargs):
             """Wrapper for client.execute(%r, %r, *params, **kwargs)."""
+            if 'context' not in kwargs:
+                kwargs['context'] = self.client.context
             return self._execute(attr, *params, **kwargs)
         wrapper.__name__ = attr
         wrapper.__doc__ %= (self._name, attr)
@@ -1126,7 +1162,7 @@ class Model(object):
 
 
 class RecordList(object):
-    """A sequence of OpenERP :class:`Record`.
+    """A sequence of Odoo :class:`Record`.
 
     It has a similar API as the :class:`Record` class, but for a list of
     records.  The attributes of the ``RecordList`` are read-only, and they
@@ -1142,6 +1178,8 @@ class RecordList(object):
             if isinstance(id_, (list, tuple)):
                 ids[index] = id_ = id_[0]
             assert isinstance(id_, int_types), repr(id_)
+        if context is None:
+            context = res_model.client.context
         # Bypass the __setattr__ method
         self.__dict__.update({
             'id': ids,
@@ -1167,9 +1205,14 @@ class RecordList(object):
     def __len__(self):
         return len(self.id)
 
+    def __add__(self, other):
+        assert self._model is other._model, 'Model mismatch'
+        ids = self._idnames + other._idnames
+        return RecordList(self._model, ids, self._context)
+
     def read(self, fields=None, context=None):
         """Wrapper for :meth:`Record.read` method."""
-        if context is None and self._context:
+        if context is None:
             context = self._context
 
         client = self._model.client
@@ -1178,7 +1221,7 @@ class RecordList(object):
                                  fields, order=True, context=context)
             if is_list_of_dict(values):
                 browse_values = self._model._browse_values
-                return [v and browse_values(v) for v in values]
+                return [v and browse_values(v, context) for v in values]
         else:
             values = []
 
@@ -1190,23 +1233,23 @@ class RecordList(object):
                     return RecordList(rel_model, values, context=context)
                 if field['type'] in ('one2many', 'many2many'):
                     rel_model = client.model(field['relation'], False)
-                    return [RecordList(rel_model, v) for v in values]
+                    return [RecordList(rel_model, v, context) for v in values]
                 if field['type'] == 'reference':
                     records = []
                     for value in values:
                         if value:
                             (res_model, res_id) = value.split(',')
                             rel_model = client.model(res_model, False)
-                            value = Record(rel_model, int(res_id))
+                            value = Record(rel_model, int(res_id), context)
                         records.append(value)
                     return records
         return values
 
     def write(self, values, context=None):
-        """Write the `values` in the :class:`RecordList`."""
+        """Wrapper for :meth:`Record.write` method."""
         if not self.id:
             return True
-        if context is None and self._context:
+        if context is None:
             context = self._context
         values = self._model._unbrowse_values(values)
         rv = self._execute('write', self.id, values, context=context)
@@ -1216,7 +1259,7 @@ class RecordList(object):
         """Wrapper for :meth:`Record.unlink` method."""
         if not self.id:
             return True
-        if context is None and self._context:
+        if context is None:
             context = self._context
         rv = self._execute('unlink', self.id, context=context)
         return rv
@@ -1250,8 +1293,8 @@ class RecordList(object):
 
         def wrapper(self, *params, **kwargs):
             """Wrapper for client.execute(%r, %r, [...], *params, **kwargs)."""
-            if context:
-                kwargs.setdefault('context', context)
+            if context is not None and 'context' not in kwargs:
+                kwargs['context'] = context
             return self._execute(attr, self.id, *params, **kwargs)
         wrapper.__name__ = attr
         wrapper.__doc__ %= (self._model_name, attr)
@@ -1267,9 +1310,9 @@ class RecordList(object):
 
 
 class Record(object):
-    """A class for all OpenERP records.
+    """A class for all Odoo records.
 
-    It maps any OpenERP object.
+    It maps any Odoo object.
     The fields can be accessed through attributes.  The changes are immediately
     sent to the server.
     The ``many2one``, ``one2many`` and ``many2many`` attributes are wrapped in
@@ -1283,6 +1326,8 @@ class Record(object):
             (res_id, res_name) = res_id
             self.__dict__['_name'] = res_name
         assert isinstance(res_id, int_types), repr(res_id)
+        if context is None:
+            context = res_model.client.context
         # Bypass the __setattr__ method
         self.__dict__.update({
             'id': res_id,
@@ -1335,7 +1380,7 @@ class Record(object):
         The argument `fields` accepts different kinds of values.
         See :meth:`Client.read` for details.
         """
-        if context is None and self._context:
+        if context is None:
             context = self._context
         rv = self._model.read(self.id, fields, context=context)
         if isinstance(rv, dict):
@@ -1350,12 +1395,18 @@ class Record(object):
         Return a dictionary of values.
         See :meth:`Client.perm_read` for details.
         """
+        if context is None:
+            context = self._context
         rv = self._execute('perm_read', [self.id], context=context)
         return rv[0] if rv else None
 
     def write(self, values, context=None):
-        """Write the `values` in the :class:`Record`."""
-        if context is None and self._context:
+        """Write the `values` in the :class:`Record`.
+
+        `values` is a dictionary of values.
+        See :meth:`Model.create` for details.
+        """
+        if context is None:
             context = self._context
         values = self._model._unbrowse_values(values)
         rv = self._execute('write', [self.id], values, context=context)
@@ -1364,7 +1415,7 @@ class Record(object):
 
     def unlink(self, context=None):
         """Delete the current :class:`Record` from the database."""
-        if context is None and self._context:
+        if context is None:
             context = self._context
         rv = self._execute('unlink', [self.id], context=context)
         self.refresh()
@@ -1376,12 +1427,12 @@ class Record(object):
         The optional argument `default` is a mapping which overrides some
         values of the new record.
         """
-        if context is None and self._context:
+        if context is None:
             context = self._context
         if default:
             default = self._model._unbrowse_values(default)
         new_id = self._execute('copy', self.id, default, context=context)
-        return Record(self._model, new_id)
+        return Record(self._model, new_id, context=context)
 
     def _send(self, signal):
         """Trigger workflow `signal` for this :class:`Record`."""
@@ -1428,8 +1479,8 @@ class Record(object):
 
         def wrapper(self, *params, **kwargs):
             """Wrapper for client.execute(%r, %r, %d, *params, **kwargs)."""
-            if context:
-                kwargs.setdefault('context', context)
+            if context is not None and 'context' not in kwargs:
+                kwargs['context'] = context
             res = self._execute(attr, [self.id], *params, **kwargs)
             self.refresh()
             if isinstance(res, list) and len(res) == 1:
@@ -1506,14 +1557,13 @@ def _interact(global_vars, use_pprint=True, usage=USAGE):
                 msg = ''.join(format_exception(exc_type, exc, tb, chain=False))
                 print(msg.strip())
 
-    warnings.simplefilter('always', UserWarning)
     sys.exc_clear() if hasattr(sys, 'exc_clear') else None  # Python 2.x
     # Key UP to avoid an empty line
     Console().interact('\033[A')
 
 
-def main():
-    description = ('Inspect data on OpenERP objects.  Use interactively '
+def main(interact=_interact):
+    description = ('Inspect data on Odoo objects.  Use interactively '
                    'or query a model (-m) and pass search terms or '
                    'ids as positional parameters after the options.')
     parser = optparse.OptionParser(
@@ -1527,10 +1577,10 @@ def main():
         '--env',
         help='read connection settings from the given section')
     parser.add_option(
-        '-c', '--config', default=CONF_FILE,
+        '-c', '--config', default=None,
         help='specify alternate config file (default: %r)' % CONF_FILE)
     parser.add_option(
-        '--server', default=DEFAULT_URL,
+        '--server', default=None,
         help='full URL to the XML-RPC server (default: %s)' % DEFAULT_URL)
     parser.add_option('-d', '--db', default=DEFAULT_DB, help='database')
     parser.add_option('-u', '--user', default=DEFAULT_USER, help='username')
@@ -1552,7 +1602,7 @@ def main():
 
     (args, domain) = parser.parse_args()
 
-    Client._config_file = os.path.join(os.path.curdir, args.config)
+    Client._config_file = os.path.join(os.curdir, args.config or CONF_FILE)
     if args.list_env:
         print('Available settings:  ' + ' '.join(read_config()))
         return
@@ -1564,12 +1614,14 @@ def main():
     if args.env:
         client = Client.from_config(args.env, verbose=args.verbose)
     else:
+        if not args.server:
+            args.server = ['-c', args.config] if args.config else DEFAULT_URL
         client = Client(args.server, args.db, args.user, args.password,
                         verbose=args.verbose)
+    client.context = {'lang': (os.getenv('LANG') or 'en_US').split('.')[0]}
 
     if args.model and domain and client.user:
-        context = {'lang': (os.environ.get('LANG') or 'en_US').split('.')[0]}
-        data = client.execute(args.model, 'read', domain, args.fields, context)
+        data = client.execute(args.model, 'read', domain, args.fields)
         if not args.fields:
             args.fields = ['id']
             if data:
@@ -1580,10 +1632,10 @@ def main():
         writer.writerows(data)
 
     if client.connect is not None:
-        # Set the globals()
-        client.connect()
+        if not client.user:
+            client.connect()
         # Enter interactive mode
-        _interact(global_vars)
+        return interact(global_vars) if interact else global_vars
 
 if __name__ == '__main__':
     main()
