@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-import mock
 from mock import sentinel, ANY
 
 import erppeek
@@ -66,6 +65,8 @@ class TestCase(XmlRpcTestCase):
                 keys = ('id', 'name', 'message', 'spam', 'birthdate', 'city')
             fields = dict.fromkeys(keys, {'type': sentinel.FIELD_TYPE})
             fields['misc_id'] = {'type': 'many2one', 'relation': 'foo.misc'}
+            fields['line_ids'] = {'type': 'one2many', 'relation': 'foo.lines'}
+            fields['many_ids'] = {'type': 'many2many', 'relation': 'foo.many'}
             return fields
         if method == 'name_get':
             if 404 in args[5]:
@@ -91,10 +92,9 @@ class TestModel(TestCase):
         # Reset cache for this test
         self.client._models.clear()
 
-        self.assertIsNone(self.client.model('mic.mac'))
-        self.assertIsNone(self.client.MicMac)
+        self.assertRaises(erppeek.Error, self.client.model, 'mic.mac')
+        self.assertRaises(erppeek.Error, getattr, self.client, 'MicMac')
         self.assertCalls(ANY, ANY, ANY, ANY)
-        self.assertIn('Model not found', self.stdout.popvalue())
         self.assertOutput('')
 
         self.assertIs(self.client.model('foo.bar'),
@@ -165,12 +165,9 @@ class TestModel(TestCase):
         )
         self.assertOutput('')
 
-        # UserWarning
-        warn = mock.patch('warnings.warn').start()
+        # No longer supported since 1.6
         FooBar.search('name like Morice')
-        self.assertCalls(OBJ('foo.bar', 'search', domain))
-        warn.assert_called_once_with(
-            "Domain should be a list: ['name like Morice']")
+        self.assertCalls(OBJ('foo.bar', 'search', 'name like Morice'))
 
         FooBar.search(['name like Morice'], missingkey=42)
         self.assertCalls(OBJ('foo.bar', 'search', domain, 0, None, None, None))
@@ -205,11 +202,9 @@ class TestModel(TestCase):
         )
         self.assertOutput('')
 
-        warn = mock.patch('warnings.warn').start()
+        # No longer supported since 1.6
         FooBar.count(searchterm)
-        self.assertCalls(OBJ('foo.bar', 'search_count', domain))
-        warn.assert_called_once_with(
-            "Domain should be a list: ['name like Morice']")
+        self.assertCalls(OBJ('foo.bar', 'search_count', searchterm))
 
         self.assertRaises(TypeError, FooBar.count,
                           [searchterm], limit=2)
@@ -282,11 +277,9 @@ class TestModel(TestCase):
         )
         self.assertOutput('')
 
-        warn = mock.patch('warnings.warn').start()
+        # No longer supported since 1.6
         FooBar.read(searchterm)
-        self.assertCalls(OBJ('foo.bar', 'search', domain), call_read())
-        warn.assert_called_once_with(
-            "Domain should be a list: ['name like Morice']")
+        self.assertCalls(OBJ('foo.bar', 'read', [searchterm], None))
 
         FooBar.read([searchterm], missingkey=42)
         self.assertCalls(OBJ('foo.bar', 'search', domain, 0, None, None, None),
@@ -334,11 +327,8 @@ class TestModel(TestCase):
         )
         self.assertOutput('')
 
-        warn = mock.patch('warnings.warn').start()
-        FooBar.browse(searchterm)
-        self.assertCalls(OBJ('foo.bar', 'search', domain))
-        warn.assert_called_once_with(
-            "Domain should be a list: ['name like Morice']")
+        # No longer supported since 1.6
+        self.assertRaises(AssertionError, FooBar.browse, searchterm)
 
         FooBar.browse([searchterm], limit=2, fields=['birthdate', 'city'])
         FooBar.browse([searchterm], missingkey=42)
@@ -366,22 +356,33 @@ class TestModel(TestCase):
         self.assertIsInstance(FooBar.get(['name = Morice']), erppeek.Record)
         self.assertIsNone(FooBar.get(['name = Blinky', 'missing = False']))
 
+        # domain matches too many records (2)
+        self.assertRaises(ValueError, FooBar.get, ['name like Morice'])
+
+        # set default context
+        ctx = {'lang': 'en_GB', 'location': 'somewhere'}
+        self.client.context = dict(ctx)
+
         # with context
         value = FooBar.get(['name = Morice'], context={'lang': 'fr_FR'})
         self.assertEqual(type(value), erppeek.Record)
         self.assertIsInstance(value.name, str)
 
-        # domain matches too many records (2)
-        self.assertRaises(ValueError, FooBar.get, ['name like Morice'])
+        # with default context
+        value = FooBar.get(['name = Morice'])
+        self.assertEqual(type(value), erppeek.Record)
+        self.assertIsInstance(value.name, str)
 
         self.assertCalls(
             OBJ('foo.bar', 'search', [('name', '=', 'Morice')]),
             OBJ('foo.bar', 'search', [('name', '=', 'Blinky'), ('missing', '=', False)]),
+            OBJ('foo.bar', 'search', [('name', 'like', 'Morice')]),
             OBJ('foo.bar', 'search', [('name', '=', 'Morice')], 0, None, None, {'lang': 'fr_FR'}),
             OBJ('foo.bar', 'fields_get_keys'),
             OBJ('foo.bar', 'read', [1003], ['name'], {'lang': 'fr_FR'}),
             OBJ('foo.bar', 'fields_get'),
-            OBJ('foo.bar', 'search', [('name', 'like', 'Morice')]),
+            OBJ('foo.bar', 'search', [('name', '=', 'Morice')], 0, None, None, ctx),
+            OBJ('foo.bar', 'read', [1003], ['name'], ctx),
         )
         self.assertOutput('')
 
@@ -441,6 +442,52 @@ class TestModel(TestCase):
         )
         self.assertOutput('')
 
+    def test_create_relation(self):
+        FooBar = self.model('foo.bar')
+
+        record42 = FooBar.browse(42)
+        recordlist42 = FooBar.browse([4, 2])
+        rec_null = FooBar.browse(False)
+
+        # one2many
+        FooBar.create({'line_ids': rec_null})
+        FooBar.create({'line_ids': []})
+        FooBar.create({'line_ids': [123, 234]})
+        FooBar.create({'line_ids': [(6, 0, [76])]})
+        FooBar.create({'line_ids': recordlist42})
+
+        # many2many
+        FooBar.create({'many_ids': None})
+        FooBar.create({'many_ids': []})
+        FooBar.create({'many_ids': [123, 234]})
+        FooBar.create({'many_ids': [(6, 0, [76])]})
+        FooBar.create({'many_ids': recordlist42})
+
+        # many2one
+        FooBar.create({'misc_id': False})
+        FooBar.create({'misc_id': 123})
+        FooBar.create({'misc_id': record42})
+
+        self.assertCalls(
+            OBJ('foo.bar', 'fields_get'),
+            OBJ('foo.bar', 'create', {'line_ids': [(6, 0, [])]}),
+            OBJ('foo.bar', 'create', {'line_ids': [(6, 0, [])]}),
+            OBJ('foo.bar', 'create', {'line_ids': [(6, 0, [123, 234])]}),
+            OBJ('foo.bar', 'create', {'line_ids': [(6, 0, [76])]}),
+            OBJ('foo.bar', 'create', {'line_ids': [(6, 0, [4, 2])]}),
+
+            OBJ('foo.bar', 'create', {'many_ids': [(6, 0, [])]}),
+            OBJ('foo.bar', 'create', {'many_ids': [(6, 0, [])]}),
+            OBJ('foo.bar', 'create', {'many_ids': [(6, 0, [123, 234])]}),
+            OBJ('foo.bar', 'create', {'many_ids': [(6, 0, [76])]}),
+            OBJ('foo.bar', 'create', {'many_ids': [(6, 0, [4, 2])]}),
+
+            OBJ('foo.bar', 'create', {'misc_id': False}),
+            OBJ('foo.bar', 'create', {'misc_id': 123}),
+            OBJ('foo.bar', 'create', {'misc_id': 42}),
+        )
+        self.assertOutput('')
+
     def test_method(self, method_name='method', single_id=True):
         FooBar = self.model('foo.bar')
         FooBar_method = getattr(FooBar, method_name)
@@ -489,9 +536,11 @@ class TestRecord(TestCase):
     def test_read(self):
         records = self.model('foo.bar').browse([13, 17])
         rec = self.model('foo.bar').browse(42)
+        rec_null = self.model('foo.bar').browse(False)
 
         self.assertIsInstance(records, erppeek.RecordList)
         self.assertIsInstance(rec, erppeek.Record)
+        self.assertIsInstance(rec_null, erppeek.Record)
 
         rec.read()
         records.read()
@@ -536,6 +585,94 @@ class TestRecord(TestCase):
         )
         self.assertOutput('')
 
+    def test_write_relation(self):
+        records = self.model('foo.bar').browse([13, 17])
+        rec = self.model('foo.bar').browse(42)
+        rec_null = self.model('foo.bar').browse(False)
+
+        # one2many
+        rec.write({'line_ids': False})
+        rec.write({'line_ids': []})
+        rec.write({'line_ids': [123, 234]})
+        rec.write({'line_ids': [(6, 0, [76])]})
+        rec.write({'line_ids': records})
+
+        # many2many
+        rec.write({'many_ids': None})
+        rec.write({'many_ids': []})
+        rec.write({'many_ids': [123, 234]})
+        rec.write({'many_ids': [(6, 0, [76])]})
+        rec.write({'many_ids': records})
+
+        # many2one
+        rec.write({'misc_id': False})
+        rec.write({'misc_id': 123})
+        rec.write({'misc_id': rec})
+
+        # one2many
+        records.write({'line_ids': None})
+        records.write({'line_ids': []})
+        records.write({'line_ids': [123, 234]})
+        records.write({'line_ids': [(6, 0, [76])]})
+        records.write({'line_ids': records})
+
+        # many2many
+        records.write({'many_ids': 0})
+        records.write({'many_ids': []})
+        records.write({'many_ids': [123, 234]})
+        records.write({'many_ids': [(6, 0, [76])]})
+        records.write({'many_ids': records})
+
+        # many2one
+        records.write({'misc_id': rec_null})
+        records.write({'misc_id': 123})
+        records.write({'misc_id': rec})
+
+        self.assertCalls(
+            OBJ('foo.bar', 'fields_get'),
+
+            OBJ('foo.bar', 'write', [42], {'line_ids': [(6, 0, [])]}),
+            OBJ('foo.bar', 'write', [42], {'line_ids': [(6, 0, [])]}),
+            OBJ('foo.bar', 'write', [42], {'line_ids': [(6, 0, [123, 234])]}),
+            OBJ('foo.bar', 'write', [42], {'line_ids': [(6, 0, [76])]}),
+            OBJ('foo.bar', 'write', [42], {'line_ids': [(6, 0, [13, 17])]}),
+
+            OBJ('foo.bar', 'write', [42], {'many_ids': [(6, 0, [])]}),
+            OBJ('foo.bar', 'write', [42], {'many_ids': [(6, 0, [])]}),
+            OBJ('foo.bar', 'write', [42], {'many_ids': [(6, 0, [123, 234])]}),
+            OBJ('foo.bar', 'write', [42], {'many_ids': [(6, 0, [76])]}),
+            OBJ('foo.bar', 'write', [42], {'many_ids': [(6, 0, [13, 17])]}),
+
+            OBJ('foo.bar', 'write', [42], {'misc_id': False}),
+            OBJ('foo.bar', 'write', [42], {'misc_id': 123}),
+            OBJ('foo.bar', 'write', [42], {'misc_id': 42}),
+
+            OBJ('foo.bar', 'write', [13, 17], {'line_ids': [(6, 0, [])]}),
+            OBJ('foo.bar', 'write', [13, 17], {'line_ids': [(6, 0, [])]}),
+            OBJ('foo.bar', 'write', [13, 17], {'line_ids': [(6, 0, [123, 234])]}),
+            OBJ('foo.bar', 'write', [13, 17], {'line_ids': [(6, 0, [76])]}),
+            OBJ('foo.bar', 'write', [13, 17], {'line_ids': [(6, 0, [13, 17])]}),
+
+            OBJ('foo.bar', 'write', [13, 17], {'many_ids': [(6, 0, [])]}),
+            OBJ('foo.bar', 'write', [13, 17], {'many_ids': [(6, 0, [])]}),
+            OBJ('foo.bar', 'write', [13, 17], {'many_ids': [(6, 0, [123, 234])]}),
+            OBJ('foo.bar', 'write', [13, 17], {'many_ids': [(6, 0, [76])]}),
+            OBJ('foo.bar', 'write', [13, 17], {'many_ids': [(6, 0, [13, 17])]}),
+
+            OBJ('foo.bar', 'write', [13, 17], {'misc_id': False}),
+            OBJ('foo.bar', 'write', [13, 17], {'misc_id': 123}),
+            OBJ('foo.bar', 'write', [13, 17], {'misc_id': 42}),
+        )
+
+        self.assertRaises(TypeError, rec.write, {'line_ids': 123})
+        self.assertRaises(TypeError, records.write, {'line_ids': 123})
+        self.assertRaises(TypeError, records.write, {'line_ids': rec})
+        self.assertRaises(TypeError, rec.write, {'many_ids': 123})
+        self.assertRaises(TypeError, records.write, {'many_ids': rec})
+
+        self.assertCalls()
+        self.assertOutput('')
+
     def test_copy(self):
         rec = self.model('foo.bar').browse(42)
         records = self.model('foo.bar').browse([13, 17])
@@ -692,6 +829,37 @@ class TestRecord(TestCase):
         self.assertCalls()
         self.assertOutput('')
 
+    def test_add(self):
+        records1 = self.model('foo.bar').browse([42])
+        records2 = self.model('foo.bar').browse([42])
+        records3 = self.model('foo.bar').browse([13, 17])
+        records4 = self.model('foo.other').browse([4])
+        rec1 = self.model('foo.bar').get(42)
+
+        sum1 = records1 + records2
+        sum2 = records1 + records3
+        sum3 = records3
+        sum3 += records1
+        self.assertIsInstance(sum1, erppeek.RecordList)
+        self.assertIsInstance(sum2, erppeek.RecordList)
+        self.assertIsInstance(sum3, erppeek.RecordList)
+        self.assertEqual(sum1.id, [42, 42])
+        self.assertEqual(sum2.id, [42, 13, 17])
+        self.assertEqual(sum3.id, [13, 17, 42])
+        self.assertEqual(records3.id, [13, 17])
+
+        with self.assertRaises(AssertionError):
+            records1 + records4
+        with self.assertRaises(AttributeError):
+            records1 + rec1
+        with self.assertRaises(AttributeError):
+            records1 + [rec1]
+        with self.assertRaises(TypeError):
+            rec1 + rec1
+
+        self.assertCalls(OBJ('foo.bar', 'fields_get_keys'))
+        self.assertOutput('')
+
     def test_read_duplicate(self):
         records = self.model('foo.bar').browse([17, 17])
 
